import * as React from "react";
import {observer} from "mobx-react-lite";
import {Box, Container, Grid, makeStyles, Theme, Typography} from "@material-ui/core";
import {ArBatchManager, ArBatchRunData, DomeSide} from "../../../../objects/batch_workstation_manager/ArBatchManager";
import {ButtonField, Conditional, InputField, PaddedPaper, SelectField} from "@rx-suite/ui";
import {t, Trans} from "@lingui/macro";
import {GroupWork, PlayArrow} from "@material-ui/icons";
import {grey} from "@material-ui/core/colors";
import {useBatchManagerContext} from "../../../hooks/useBatchManagerContext";
import {LoaderWithText} from "../../../generic/LoaderWithText";
import {GenerateBatchIdButton} from "../../../batch_workstations/toolbar/GenerateBatchIdButton";
import {useMemo} from "react";
import {BoxCoaterRecipeModel, BoxCoaterRecipeStore} from "../../../../stores/BoxCoaterRecipeStore";

const useStyles = makeStyles((theme: Theme) => ({
        header: {
            color: grey[700],
        },
        headerText: {
            fontSize: '2.5rem',
            textAlign: 'center'
        },
        headerIcon: {
            fontSize: '2.5rem',
            marginRight: theme.spacing(1)
        },
        paddedPaper: {
            padding: theme.spacing(4)
        },
        batchIdField: {
            flexGrow: 1
        }
    })
);

const domeSideOptions = [
    {label: 'Convex and Concave (CC & CX)', value: 'both'},
    {label: 'Only Concave (CC)', value: 'CC'},
    {label: 'Only Convex (CX)', value: 'CX'},
]

export const ArBatchNew: React.FunctionComponent = observer(({}) => {
    const classes = useStyles();
    const manager = useBatchManagerContext() as ArBatchManager;

    BoxCoaterRecipeStore.getAll();

    const [tempBatchId, setTempBatchId] = React.useState<string>(manager.getNewBatchId());
    const [numberOfSectors, setNumberOfSectors] = React.useState<number>(3);
    const [domeSide, setDomeSide] = React.useState<DomeSide>('both');
    const [concaveRecipeCode, setConcaveRecipeCode] = React.useState<string | ''>('');
    const [convexRecipeCode, setConvexRecipeCode] = React.useState<string | ''>('');
    const [isSubmitting, setIsSubmitting] = React.useState<boolean>(false);

    const numberOfSectorsOptions = React.useMemo(() => {
        return [
            {label: '3', value: 3},
            {label: '6', value: 6}
        ];
    }, []);

    const concaveRecipeOptions = React.useMemo(() => {
        return BoxCoaterRecipeStore.models.filter(recipe => recipe.isConcaveRecipe).map(recipe => (
            {label: recipe.name, value: recipe.recipe_code}
        ));
    }, [BoxCoaterRecipeStore.models]);

    const convexRecipeOptions = React.useMemo(() => {
        return BoxCoaterRecipeStore.models.filter(recipe => recipe.isConvexRecipe).map(recipe => (
            {label: recipe.name, value: recipe.recipe_code}
        ));
    }, [BoxCoaterRecipeStore.models]);

    const requireConvex = React.useMemo(() => {
        return domeSide === 'CX' || domeSide === 'both';
    }, [domeSide])

    const requireConcave = React.useMemo(() => {
        return domeSide === 'CC' || domeSide === 'both';
    }, [domeSide])

    const formIsValid = useMemo(()=> {
        const dataToCheck = [tempBatchId];

        if(requireConcave && concaveRecipeCode.trim() === ''){
            return false;
        }

        if(requireConvex && convexRecipeCode.trim() === ''){
            return false;
        }

        return !(dataToCheck.some(val => val === null));
    }, [tempBatchId, concaveRecipeCode, convexRecipeCode, requireConvex, requireConcave]);

    const handleChangeRequiredSide = React.useCallback((val: DomeSide) => {
        setConcaveRecipeCode('');
        setConvexRecipeCode('');
        setDomeSide(val);
    }, []);

    const handleCreateBatch = async () => {
        if(!formIsValid){
            return;
        }

        setIsSubmitting(true);

        const runs: ArBatchRunData[] = [];

        if(requireConvex){
            runs.push({
                run_key: '', // let the batch manager generate this
                recipeName: (BoxCoaterRecipeStore.models.find(recipe => recipe.recipe_code === convexRecipeCode) as BoxCoaterRecipeModel).name,
                recipeCode: convexRecipeCode,
                domeSide: 'CX'
            });
        }

        if(requireConcave){
            runs.push({
                run_key: '',
                recipeName: (BoxCoaterRecipeStore.models.find(recipe => recipe.recipe_code === concaveRecipeCode) as BoxCoaterRecipeModel).name,
                recipeCode: concaveRecipeCode,
                domeSide: 'CC'
            });
        }

        await manager.initialize({
            batchId: tempBatchId as string,
            runs: runs,
            sectorCount: numberOfSectors
        });
    }

    if(manager.isLoadingSettings){
        return (<LoaderWithText message={t`Loading new AR Batch Settings`}/>);
    }

    return (
        <Container maxWidth={'sm'}>
            <PaddedPaper elevation={2} className={classes.paddedPaper}>
                <Box mb={4}>
                    <Grid container alignItems={'center'} justifyContent={'center'} className={classes.header}>
                        <GroupWork className={classes.headerIcon}/>
                        <Typography className={classes.headerText}>
                            <Trans>New AR Coating Batch</Trans>
                        </Typography>
                    </Grid>
                </Box>
                <Box mb={4}>
                    <Grid container alignItems={'center'}>
                        <InputField
                            key={'batch_id'}
                            label={t`Batch ID`}
                            value={tempBatchId}
                            className={classes.batchIdField}
                            onChange={(e) => { setTempBatchId(e.target.value); }}
                        />
                        <GenerateBatchIdButton setTempId={setTempBatchId}/>
                    </Grid>
                </Box>
                <SelectField
                    key={'sector_count'}
                    label={'Number of Sectors'}
                    value={numberOfSectors}
                    options={numberOfSectorsOptions || []}
                    onChange={(name, value: number) => { setNumberOfSectors(value); }}
                />
                <SelectField
                    key={'lens_side'}
                    label={t`Dome Lens Side`}
                    value={domeSide}
                    options={domeSideOptions}
                    onChange={(name, value: DomeSide) => { handleChangeRequiredSide(value)}}
                />
                <Conditional condition={requireConcave}>
                    <SelectField
                        key={'concave_recipe'}
                        label={t`Concave Side Recipe`}
                        onChange={(name, value) => {setConcaveRecipeCode(value)}}
                        value={concaveRecipeCode}
                        options={concaveRecipeOptions}
                        className={classes.batchIdField}
                    />
                </Conditional>
                <Conditional condition={requireConvex}>
                    <SelectField
                        key={'convex_recipe'}
                        label={t`Convex Side Recipe`}
                        onChange={(name, value) => {setConvexRecipeCode(value)}}
                        value={convexRecipeCode}
                        options={convexRecipeOptions}
                        className={classes.batchIdField}
                    />
                </Conditional>
                <Grid container justifyContent={'center'}>
                    <Box mt={4}>
                        <ButtonField
                            startIcon={<PlayArrow/>}
                            text={t`Start New Batch`}
                            color={'secondary'}
                            variant={'contained'}
                            size={'large'}
                            disabled={!formIsValid}
                            loading={isSubmitting}
                            onClick={handleCreateBatch}
                        />
                    </Box>
                </Grid>
            </PaddedPaper>
        </Container>
    );
});