import * as React from "react";
import {observer, Observer} from "mobx-react-lite";
import ReactTable from "react-table";
import {t} from "@lingui/macro";
import {Box, Typography} from "@material-ui/core";
import {Clear, SkipNext} from "@material-ui/icons";
import {makeStyles} from "@material-ui/core/styles";
import {LensDatum} from "../../../../objects/ar_coating/LensDatum";
import {useBatchManagerContext} from "../../../hooks/useBatchManagerContext";
import {ArBatchManager} from "../../../../objects/batch_workstation_manager/ArBatchManager";
import {ButtonField, Conditional, IfFalse, IfTrue} from "@rx-suite/ui";
import {grey, orange} from "@material-ui/core/colors";
import classNames from "classnames";

const useStyles = makeStyles({
    actionButtons: {
        '& > *': {
            marginLeft: '0.5rem',
            marginRight: '0.5rem'
        }
    },
    cellWrapper: {
        width: '100%',
        height: '100%',
        display: 'flex',
        alignItems: 'center',
        justifyContent: 'center',
    },
    skipped: {
        backgroundColor: grey[300]
    },
    selected: {
        backgroundColor: orange[100]
    },
    buttonMargin: {
        marginTop: '4px',
        marginBottom: '4px'
    }
});

export const ArSectorEditLensTable: React.FunctionComponent = observer(() => {
    const classes = useStyles();

    const manager = useBatchManagerContext() as ArBatchManager;
    const [focusedPage, setFocusedPage] = React.useState<number|undefined>(undefined);

    const resolvedCellClasses = (data: LensDatum) => {
        const resolvedClasses = [classes.cellWrapper];

        if(manager.currentSector.lastUsedLensPosition.includes(data.position)){
            resolvedClasses.push(classes.selected)
        }
        else if(data.skipped){
            resolvedClasses.push(classes.skipped)
        }

        return classNames(resolvedClasses);
    }

    React.useEffect(() => {
        if(manager.currentSector.lastUsedLensPosition.length === 0){
            setFocusedPage(undefined);
        }else{
            setFocusedPage(Math.floor(manager.currentSector.lastUsedLensPosition[0] / 10));
        }
    }, [manager.currentSector.lastUsedLensPosition.length]);

    React.useEffect(() => {
        // On the first load, jump straight to the page with the next available hole
        const nextAvailableLensHole = manager.currentSector.nextAvailableHolePosition;
        setFocusedPage(Math.floor((nextAvailableLensHole === -1 ? manager.currentSector.lensData.length - 1 : nextAvailableLensHole) / 10));
    }, [manager.currentSector]);

    const columns = [
        {
            Header: t`Lens Position`,
            id: 'lens_position',
            accessor: (data: LensDatum) => data,
            Cell: (cellInfo: { value: LensDatum }) => {
                return (
                    <Observer>
                        {
                            () => {
                                return (
                                    <div className={resolvedCellClasses(cellInfo.value)}>
                                        <Typography variant={'body1'}>{cellInfo.value.position}</Typography>
                                    </div>
                                )
                            }
                        }
                    </Observer>
                )
            }
        },
        {
            Header: t`Job Number`,
            id: '_jobNumber',
            accessor: (data: LensDatum) => data,
            Cell: (cellInfo: { value: LensDatum }) => {
                return (
                    <Observer>
                        {
                            () => {
                                const lensData = cellInfo.value;
                                let text = '';

                                if (lensData.skipped) {
                                    text = 'skipped';
                                }

                                if (lensData.jobInfo) {
                                    text = lensData.jobInfo.jobNumber;
                                }

                                return (
                                    <div className={resolvedCellClasses(cellInfo.value)}>
                                        <Typography variant={'body1'}>{text}</Typography>
                                    </div>
                                )
                            }
                        }
                    </Observer>
                )
            }
        },
        {
            Header: t`Side`,
            id: '_side',
            accessor: (data: LensDatum) => data,
            Cell: (cellInfo: { value: LensDatum }) => {
                return (
                    <Observer>
                        {
                            () => {

                                const lensData = cellInfo.value;
                                let text = '';

                                if (lensData.skipped) {
                                    text = 'skipped';
                                }

                                if (lensData.jobInfo) {
                                    text = lensData.jobInfo.side;
                                }

                                return (
                                    <div className={resolvedCellClasses(cellInfo.value)}>
                                        <Typography variant={'body1'}>{text}</Typography>
                                    </div>
                                )
                            }
                        }
                    </Observer>
                )
            }
        },
        {
            accessor: (data: LensDatum) => data,
            id: '_actions',
            style: {textAlign: "right"},
            minWidth: 125,
            Cell: (cellInfo: { value: LensDatum }) => {
                return (
                    <Observer>
                        {
                            () => {
                               return (
                                   <div className={resolvedCellClasses(cellInfo.value)}>
                                       <Conditional condition={cellInfo.value.isAvailable}>
                                           <IfTrue>
                                               <ButtonField
                                                   text={t`Skip`}
                                                   variant="contained"
                                                   color="primary"
                                                   endIcon={<SkipNext/>}
                                                   disabled={manager.controlsDisabled}
                                                   onClick={() => { manager.callLensAction(() => cellInfo.value.skip())} }
                                                   size={'small'}
                                                   className={classes.buttonMargin}
                                               />
                                           </IfTrue>
                                           <IfFalse>
                                               <ButtonField
                                                   text={t`Clear`}
                                                   variant="contained"
                                                   color="primary"
                                                   endIcon={<Clear/>}
                                                   disabled={manager.controlsDisabled}
                                                   onClick={() => { manager.callLensAction(() => cellInfo.value.clear())} }
                                                   size={'small'}
                                                   className={classes.buttonMargin}
                                               />
                                           </IfFalse>
                                       </Conditional>
                                   </div>
                               )
                            }
                        }
                    </Observer>
                )
            }
        }
    ];

    return (
        <Box mt={2}>
            <ReactTable
                data={manager.currentSectorLensData}
                columns={columns}
                minRows={10}
                defaultPageSize={10}
                page={focusedPage}
                onPageChange={() => setFocusedPage(undefined)}
                showPageSizeOptions={false}
                defaultSorted={[{ id: 'position', desc: false}]}
                sortable={false}
                getTdProps={() => {
                    return {
                        style: {
                            padding: 0
                        }
                    }
                }}
            />
        </Box>
    )
});