import * as React from "react";
import {observer} from "mobx-react-lite";
import {ArBatchManager} from "../../../../objects/batch_workstation_manager/ArBatchManager";
import {useBatchManagerContext} from "../../../hooks/useBatchManagerContext";
import {ButtonField, Conditional, IfFalse, IfTrue, SelectField} from "@rx-suite/ui";
import {Box, CircularProgress, Grid, makeStyles, Theme, Typography} from "@material-ui/core";
import {green, grey} from "@material-ui/core/colors";
import {t, Trans} from "@lingui/macro";
import {Check} from "@material-ui/icons";
import {ArSectorDomePositionCanvas} from "../../ArDomeSectorPositionCanvas";
import {BatchContainerViewCanvas} from "../../../batch_container/BatchContainerViewCanvas";
import {BatchContainerModel} from "../../../../stores/BatchContainerStore";

const useStyles = makeStyles((theme: Theme) => ({
        button: {
            backgroundColor: green[600]
        },
        sectorPositionWrap: {
            height: '5rem',
            width: '5rem'
        },
        sectorPreviewWrap: {
            marginTop: theme.spacing(4),
            marginBottom: theme.spacing(4),
            width: '30rem',
            height: '23rem',
        },
        previewEmpty: {
            width: '100%',
            height: '100%',
            border: `1px solid ${grey[300]}`,
            color: grey[600]
        },
        selectTemplateSection: {
            marginLeft: theme.spacing(2),
            width: 'fit-content'
        },
        selectFieldWrap: {
            width: '300px'
        }
    })
);


export const ArSectorSelectForm: React.FunctionComponent = observer(({}) => {
    const classes = useStyles();

    const manager = useBatchManagerContext() as ArBatchManager;

    const [isLoadingContainer, setIsLoadingContainer] = React.useState<boolean>(false);
    const [selectedContainerModel, setSelectedContainerModel] = React.useState<null | BatchContainerModel>(null);

     React.useEffect(() => {
         setSelectedContainerModel(null);
    }, [manager.currentSector]);

    const sectorOptions = React.useMemo(() => {
        return manager.availableContainers.map(container => (
            {label: container.name, value: container}
        ));
    }, [manager.availableContainers]);

    const handleChangeSectorTemplate = React.useCallback(async (container: BatchContainerModel) => {
        setIsLoadingContainer(true);

        // Set the unloaded container so that the select field responds to the user change
        setSelectedContainerModel(container);

        setSelectedContainerModel(await container.fetchFullModel());

        setIsLoadingContainer(false);
    }, []);

    const handleSelectSector = async () => {
        manager.currentSector.setTemplate(selectedContainerModel as BatchContainerModel);
        await manager.saveCurrentBatchData();
    }

    return (
        <React.Fragment>
            <Box mb={2}>
                <Typography variant={'h5'}>
                    {`Please select a template for Sector ${manager.currentSector.order}`}
                </Typography>
            </Box>
            <Grid container>
                <Box className={classes.sectorPositionWrap}>
                    <ArSectorDomePositionCanvas numberOfSectors={manager.sectorCount} highlightedSector={manager.currentSector.order-1}/>
                </Box>
                <Grid container direction={'column'} className={classes.selectTemplateSection}>
                    <Grid container justifyContent={'center'}>
                        <Box className={classes.selectFieldWrap}>
                            <SelectField
                                key={'sector_container_id'}
                                label={'Sector Template'}
                                value={selectedContainerModel}
                                options={sectorOptions || []}
                                disabled={isLoadingContainer}
                                onChange={(name, value: BatchContainerModel) => { handleChangeSectorTemplate(value) }}
                            />
                        </Box>
                        <ButtonField
                            startIcon={<Check/>}
                            text={t`Select Template`}
                            className={classes.button}
                            size={'large'}
                            disabled={isLoadingContainer || selectedContainerModel === null || manager.controlsDisabled}
                            onClick={handleSelectSector}
                        />
                    </Grid>
                    <Grid container>
                        <Grid container className={classes.sectorPreviewWrap} alignItems={'center'} justifyContent={'center'}>
                            <Conditional condition={selectedContainerModel !== null}>
                                <IfTrue>
                                    <Conditional condition={isLoadingContainer}>
                                        <IfTrue>
                                            <CircularProgress
                                                size={'5rem'}
                                            />
                                        </IfTrue>
                                        <IfFalse>
                                            <BatchContainerViewCanvas model={selectedContainerModel as BatchContainerModel}/>
                                        </IfFalse>
                                    </Conditional>
                                </IfTrue>
                                <IfFalse>
                                    <Grid container className={classes.previewEmpty} alignItems={'center'} justifyContent={'center'}>
                                        <Typography variant={'subtitle1'}>
                                            <Trans>
                                                No Sector Selected
                                            </Trans>
                                        </Typography>
                                    </Grid>
                                </IfFalse>
                            </Conditional>
                        </Grid>
                    </Grid>
                </Grid>
            </Grid>
        </React.Fragment>
    );
});