import {createStyles, Divider, makeStyles, Theme, Typography} from "@material-ui/core";
import {t, Trans} from "@lingui/macro";
import * as React from "react";
import {observer} from "mobx-react-lite";
import {
    ButtonField,
    Conditional,
    IfFalse,
    IfTrue,
    InputField,
    ModelInput,
    ModelProvider,
    SelectField
} from "@rx-suite/ui";
import {Model} from "@rx-suite/data";
import {red} from "@material-ui/core/colors";
import {BatchContainerComponentModel} from "../../../stores/BatchContainerComponentStore";
import {BatchContainerEditRenderer} from "../../../objects/canvas_renderer/batch_container/BatchContainerEditRenderer";
import {BatchContainerViewRenderer} from "../../../objects/canvas_renderer/batch_container/BatchContainerViewRender";

export type LayoutConfigComponentDetailsProps = {
    renderer: BatchContainerEditRenderer;
};

const useStyles = makeStyles((theme: Theme) =>
    createStyles({
        divider: {
            marginTop: theme.spacing(1),
            marginBottom: theme.spacing(1)
        },
        delete: {
            backgroundColor: red[500]
        }
    })
);

export const BatchConfigComponentDetails: React.FunctionComponent<LayoutConfigComponentDetailsProps> = observer(({renderer}) => {
    const model = renderer.selectedComponent;
    const classes = useStyles();

    const componentSelectOptions = React.useCallback(() => {
        if (!renderer.layout) {
            return [];
        }
        return renderer.layout.current_components.sort((a,b) =>
            (a.position === null ? 0 : a.position) - (b.position === null ? 0 : b.position)
        ).map(c => {
            return {label: c.name, value: c.viewModel}
        })
    }, [renderer.layout]);

    const handleComponentSelect = React.useCallback((n: string | undefined, component: BatchContainerComponentModel) => {
        renderer.setSelectedComponent(component);
    }, [renderer]);

    const handleDeletedSelected = React.useCallback(() => {
        renderer.deleteSelectedComponent();
    }, [renderer]);

    const handleStatusDotWidthChange = React.useCallback((e) => {
        if (!model) return;
        const val = Number(e.target.value);
        model.viewModel.width = val;
        model.viewModel.height = val;
    }, [model]);

    return (
        <React.Fragment>
            <ModelProvider value={model as Model}>
                <SelectField placeholder={t`Select component...`} onChange={handleComponentSelect}
                             value={model ? model : ""} options={componentSelectOptions()}/>
                <Divider className={classes.divider}/>
                <Conditional condition={!model}>
                    <IfTrue>
                        <Typography variant={'h6'}>No component selected.</Typography>
                    </IfTrue>
                    <IfFalse>
                        <ModelInput
                            name={'x'}
                            type={'number'}
                            inputProps={{step: 1}}
                        />
                        <ModelInput
                            name={'y'}
                            type={'number'}
                            inputProps={{step: 1}}
                        />
                        <Conditional condition={model?.viewModel.component_type === "machine_status_dot"}>
                            <IfTrue>
                                <InputField label={'Width'} onChange={handleStatusDotWidthChange}
                                            value={model ? model.viewModel.width : ''}/>
                            </IfTrue>
                            <IfFalse>
                                <ModelInput
                                    name={'width'}
                                    type={'number'}
                                    inputProps={{min: BatchContainerViewRenderer.BATCH_POSITION_MIN_WIDTH, step: 1}}
                                />
                                <ModelInput
                                    name={'height'}
                                    type={'number'}
                                    inputProps={{min: BatchContainerViewRenderer.BATCH_POSITION_MIN_HEIGHT, step: 1}}
                                />
                            </IfFalse>
                        </Conditional>
                        <ModelInput label={t`Layer`}
                                    name={'z_index'}
                                    type={'number'}
                                    inputProps={{min: 0, step: 1}}
                                    hint={{
                                        title: t`Layer`,
                                        text: t`Order in which components are placed on the layout. A component with greater layer is always in front of an element with a lower layer.`
                                    }}/>
                        <Divider className={classes.divider}/>
                        <ButtonField text={t`Delete Component`} className={classes.delete}
                                     onClick={handleDeletedSelected}/>
                    </IfFalse>
                </Conditional>
            </ModelProvider>
        </React.Fragment>
    )
});