import * as React from "react";
import {ButtonField, ScannerField} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {t} from '@lingui/macro';
import {Grid, makeStyles} from "@material-ui/core";
import {Add} from "@material-ui/icons";
import {useBatchManagerContext} from "../hooks/useBatchManagerContext";
import {useSnackbar} from "notistack";
import {InformationDialog} from "./dialogs/InformationalDialog";

interface AddJobToBatchInputFieldProps {
    disabled: boolean;
    inputRef: React.MutableRefObject<HTMLInputElement>;
}

const useStyles = makeStyles((theme) => ({
    scanField: {
        flexGrow: 1
    }
}));

export const AddJobToBatchInputField: React.FunctionComponent<AddJobToBatchInputFieldProps> = observer(({disabled, inputRef}) => {
    const classes = useStyles();

    const manager = useBatchManagerContext();

    const {enqueueSnackbar} = useSnackbar();

    const [searchFieldValue, setSearchFieldValue] = React.useState<string>("");
    const [isFetching, setIsFetching] = React.useState(false);
    const [addToBatchError, setAddToBatchError]  = React.useState<null|string>(null);

    const handleChange = (e: any) => {
        setSearchFieldValue(e.target.value);
    };

    const handleSubmit = async () => {
        if (searchFieldValue.trim() === '') {
            return;
        }
        setIsFetching(true);

        try {
            const addJobError = await manager.addJobNumberToBatch(searchFieldValue);
            enqueueSnackbar(`${t`Job Found`}`, {variant: "success"})

            if(addJobError){
                setAddToBatchError(addJobError);
            }
        }catch (e){
            console.error(e.error);
            if(e.jobNotFound){
                enqueueSnackbar(`${t`Job does not exist!`}`, {variant: "error"})
            } else {
                enqueueSnackbar(`${t`Error while adding job to batch!`}`, {variant: "error"})
            }
        }
        setSearchFieldValue("");
        setIsFetching(false);

        if(inputRef.current){
            inputRef.current.focus()
        }
    }

    return (
        <React.Fragment>
            <Grid container justifyContent={'space-evenly'} alignItems={'center'}>
                <ScannerField
                    inputConfig={{
                        label: 'Scan or Enter Job Number',
                        className: classes.scanField,
                        value: searchFieldValue,
                        onChange: handleChange,
                        disabled: manager.controlsDisabled || disabled,
                        autoFocus: true
                    }}
                    scanConfig={{
                        onComplete: handleSubmit,
                        onPartial: async (code: string) => {
                            if (code !== searchFieldValue) return;
                            await handleSubmit()
                        },
                        minLength: 4,
                        averageWaitTime: 20
                    }}
                    inputRefOverride={inputRef}
                    global // overwrites onEnterKey to be scanConfig.onComplete or scanConfig.onPartial
                />
                <ButtonField
                    text={t`Add to Batch`}
                    startIcon={<Add/>}
                    buttonType='submit'
                    onClick={handleSubmit}
                    disabled={manager.controlsDisabled || disabled}
                    loading={isFetching}
                    variant='contained'
                    size='large'
                    color='secondary'
                />
            </Grid>
            <InformationDialog message={addToBatchError} open={addToBatchError !== null} handleClose={()=>{setAddToBatchError(null)}}/>
        </React.Fragment>
    );
});