import * as React from "react";
import {observer} from "mobx-react-lite";
import {Box, Container, Grid, LinearProgress, makeStyles, Typography} from "@material-ui/core";
import {useBatchManagerContext} from "../../hooks/useBatchManagerContext";
import {useSnackbar} from "notistack";
import {t, Trans} from "@lingui/macro";
import {ButtonField, Conditional, IfFalse, IfTrue, PaddedPaper} from "@rx-suite/ui";
import classNames from "classnames";
import {ArrowBack, Assignment, CheckCircle, Error, PlayArrow} from "@material-ui/icons";
import {green, grey, red} from "@material-ui/core/colors";
import {BatchManager} from "../../../objects/batch_workstation_manager/BatchManager";
import {LoaderWithText} from "../../generic/LoaderWithText";
import {routerStore} from "@rx-suite/data";

const useStyles = makeStyles((theme) => ({
    paddedPaper: {
        padding: theme.spacing(4)
    },
    icon: {
        fontSize: '7.5rem',
        color: green[900]
    },
    text: {
        textAlign: 'center'
    },
    primary: {
        fontSize: '2.315rem'
    },
    secondary: {
        fontSize: '1.9rem'
    },
    sub: {
        fontSize: '1.5rem',
        color: grey[500]
    },
    errorInfoBox: {
        padding: theme.spacing(1),
        width: '100%',
        border: `solid 1px ${grey[300]}`,
        height: '200px',
        overflowY: 'scroll'
    }
}));

export const BatchWorkstationCompletedForm: React.FunctionComponent = observer(() => {
    const classes = useStyles();

    const manager = useBatchManagerContext();
    const {enqueueSnackbar} = useSnackbar();

    const [goToNewBatch, setGoToNewBatch] = React.useState<boolean>(false);
    const [goToSummary, setGoToSummary] = React.useState<boolean>(false);

    const handleGoToNewBatch = React.useCallback(async () => {
        setGoToNewBatch(true);
        try {
            await manager.resetBatch();
            await manager.returnToEditScreen();
        }catch (e) {
            console.error(e);
            enqueueSnackbar(`${t`Error while starting new batch!`}`, {variant: "error"})

            //only release the loader on error, otherwise the form transition will handle releasing the loaded
            setGoToNewBatch(false)
        }
    }, []);

    const handleGoToSummary = React.useCallback(async () => {
        setGoToSummary(true);
        try {
            const rxDataBatch = manager.batchSubmitSuccess;
            if (!rxDataBatch) {
                routerStore.history.push(`/batches`); // can't find model, go to index page
                return;
            }

            const rxDataBatchId = JSON.parse(rxDataBatch).rx_data_batch_id;
            if (rxDataBatchId == null) {
                routerStore.history.push(`/batches`); // can't find model, go to index page
                return;
            }

            routerStore.history.push(`/batches/${rxDataBatchId}/edit`);
        } catch (e) {
            console.error(e);
            enqueueSnackbar(`${t`Error while viewing batch summary!`}`, {variant: "error"})

            //only release the loader on error, otherwise the form transition will handle releasing the loaded
            setGoToSummary(false)
        }
    }, []);

    // When go to new batch is invoked, stop rendering the component since the batch data is no longer available
    if(goToNewBatch){
        return <LoaderWithText message={t`Loading new batch...`}/>
    }

    if(goToSummary){
        return <LoaderWithText message={t`Loading batch summary...`}/>
    }

    return (
        <Container maxWidth={'sm'}>
            <PaddedPaper elevation={2} className={classes.paddedPaper}>
                <Grid container direction={'column'} alignItems={'center'}>
                    <Typography className={classNames(classes.text, classes.primary)}>
                        {t`Batch ID: ${manager.data.batchId}`}
                    </Typography>
                    <Typography className={classNames(classes.text, classes.primary)}>
                        <Trans>was successfully saved.</Trans>
                    </Typography>
                    <Box mt={2} mb={2}>
                        <CheckCircle className={classes.icon}/>
                    </Box>
                    <Box mt={2}>
                        <ButtonField
                            startIcon={<Assignment/>}
                            text={t`View Batch`}
                            size={'large'}
                            onClick={handleGoToSummary}
                            loading={goToSummary}
                            disabled={goToNewBatch}
                        />
                        <ButtonField
                            startIcon={<PlayArrow/>}
                            text={t`Start New Batch`}
                            size={'large'}
                            onClick={handleGoToNewBatch}
                            loading={goToNewBatch}
                            disabled={goToSummary}
                        />
                    </Box>
                </Grid>
            </PaddedPaper>
        </Container>
    );
});