import * as React from "react";
import {observer} from "mobx-react-lite";
import {makeStyles, Theme} from "@material-ui/core";
import {ButtonField} from "@rx-suite/ui";
import {Delete, Save} from "@material-ui/icons";
import {green, red} from "@material-ui/core/colors";
import {BatchManager} from "../../../objects/batch_workstation_manager/BatchManager";
import {t} from "@lingui/macro";
import {useSnackbar} from "notistack";
import {WarningDialog} from "../dialogs/WarningDialog";
import {useModalHandler} from "../../hooks/useModalHandler";
import {useBatchManagerContext} from "../../hooks/useBatchManagerContext";

const useStyles = makeStyles((theme: Theme) => ({
        saveButton: {
            background: green[600]
        },
        resetButton: {
            background: red[600]
        }
    })
);

export const BatchToolbarActions: React.FunctionComponent = observer(({}) => {
    const classes = useStyles();
    const {enqueueSnackbar} = useSnackbar();

    const [resetBatchDialogOpen, handleCloseResetBatchDialog, handleOpenResetBatchDialog] = useModalHandler();
    const [isSaving, setIsSaving] = React.useState<boolean>(false);
    const [isResetting, setIsResetting] = React.useState<boolean>(false);

    const manager = useBatchManagerContext() as BatchManager;

    const handleSave = React.useCallback(async () => {
        try {
            setIsSaving(true);
            await manager.saveCurrentBatchData();
            enqueueSnackbar(`${t`Save Successful!`}`, {variant: "success"})
        }catch (e) {
            console.error(e);
            enqueueSnackbar(`${t`Error while Saving!`}`, {variant: "error"})
        }
        setIsSaving(false);
    }, []);

    const handleReset = React.useCallback(async () => {
        handleCloseResetBatchDialog();
        try {
            setIsResetting(true);
            await manager.resetBatch();
        }catch (e) {
            console.error(e);
            enqueueSnackbar(`${t`Error while resetting batch!`}`, {variant: "error"})
        }
        setIsResetting(false);
    }, []);

    return (
        <React.Fragment>
            <ButtonField
                startIcon={<Save/>}
                loading={isSaving}
                disabled={manager.controlsDisabled}
                text={t`Save Batch`}
                variant={'contained'}
                className={classes.saveButton}
                onClick={handleSave}
            />
            <ButtonField
                startIcon={<Delete/>}
                loading={isResetting}
                disabled={manager.controlsDisabled}
                text={t`Restart Batch`}
                variant={'contained'}
                className={classes.resetButton}
                onClick={handleOpenResetBatchDialog}
            />
            <WarningDialog
                warningMessage={t`This action will remove all jobs currently assigned to this batch.`}
                open={resetBatchDialogOpen}
                onConfirm={handleReset}
                handleClose={handleCloseResetBatchDialog}
            />
        </React.Fragment>
    )
});

