import {action, computed, makeObservable, observable} from "mobx";
import {LensDatum, SerializedLensDatum} from "./LensDatum";
import {LensSide} from "../batch_workstation_manager/BatchManager";
import {BatchContainerPositionStates} from "../canvas_renderer/batch_container/BatchContainerViewRender";
import {ArBatchManager} from "../batch_workstation_manager/ArBatchManager";
import {BatchContainerModel} from "../../stores/BatchContainerStore";

export class SectorDatum {
    @observable
    public sectorId: string;

    @observable
    public lensData: LensDatum[];

    @observable
    public lastUsedLensPosition: number[] = [];

    @observable
    public containerId: number | null;

    public readonly order: number;

    private readonly manager: ArBatchManager;

    constructor(manager: ArBatchManager, data: SerializedSectorDatum) {
        makeObservable(this);

        this.sectorId = data.sectorId;
        this.containerId = data.containerId;
        this.order = data.order;

        this.manager = manager;

        this.lensData = data.lensData.map(lensData => new LensDatum(lensData));
    }

    public serialize(): SerializedSectorDatum {
        return {
            sectorId: this.sectorId,
            containerId: this.containerId,
            order: this.order,
            lensData: this.lensData.map(lensData => lensData.serialize())
        }
    }

    public hasEnoughEmptyHoles(quantity: number): boolean {
        let empty = 0;

        // Look for an empty hole at the end of the sector as it's more likely to be there
        for(let i = this.lensData.length - 1; i >= 0; i--){
            if(this.lensData[i].isAvailable){
                empty++;
            }

            if(empty === quantity){
                return true;
            }
        }

        return false;
    }

    @computed
    public get isInitialized(): boolean {
        return this.containerId !== null;
    }

    @computed
    public get isComplete(): boolean {
        if(!this.isInitialized){
            return false;
        }

        return this.lensData.every(lensData => lensData.skipped || lensData.jobInfo !== null);
    }

    @computed
    public get isEmpty(): boolean {
        return this.lensData.every(lensData => !lensData.skipped && lensData.jobInfo === null);
    }

    @computed
    public get completedHolesCount(): number {
        return this.completedLensHoles.length;
    }

    @computed
    public get emptyHolesCount(): number {
        return this.emptyLensHoles.length;
    }

    @computed
    public get lensDataPositionsStates(): BatchContainerPositionStates {
        return {
            focused: this.lastUsedLensPosition,
            skipped: this.skippedLensHoles.map(data=>data.position),
            completed: this.completedLensHoles.map(data=>data.position)
        }
    }

    @computed
    public get nextAvailableHolePosition(): number {
        const next = this.lensData.find(hole => hole.jobInfo === null && !hole.skipped) as LensDatum;

        if(!next) {
            return -1;
        }

        return next.position;
    }

    @computed
    public get defaultSectorId(): string {
        return `${this.manager.data.batchId}${this.order}`
    }

    @computed
    private get completedLensHoles(): LensDatum[] {
        return this.lensData.filter(data => data.jobInfo !== null);
    }

    @computed
    private get skippedLensHoles(): LensDatum[] {
        return this.lensData.filter(data => data.skipped);
    }

    @computed
    private get emptyLensHoles(): LensDatum[] {
        return this.lensData.filter(data => data.skipped);
    }

    // ACTIONS

    @action
    public skipAllRemainingHoles(): void {
        this.lensData.forEach(lensDatum => {
           lensDatum.skip();
        });
        this.clearLastUsedLensPosition();
    }

    @action
    clearLastUsedLensPosition(): void {
        this.lastUsedLensPosition = [];
    }

    @action
    setTemplate(container: BatchContainerModel): void {
        this.lensData = container.current_components
            .filter(component => component.component_type == 'batch_position')
            .sort((a, b) => (a.position as number) - (b.position as number))
            .map(component => ( new LensDatum({
                    position: component.position as number,
                    jobInfo: null,
                    skipped: false
                }))
            )

        this.containerId = container.id;
    }

    @action
    addJobToNextAvailableHole(jobNumber: string, lensSide: LensSide): void {
        const next = this.lensData.find(hole => hole.jobInfo === null && !hole.skipped) as LensDatum;

        next.jobInfo = {
            jobNumber: jobNumber,
            side: lensSide
        }

        this.lastUsedLensPosition.push(next.position);
    }

    @action
    public reset(): void {
        this.containerId = null;
        this.lensData = [];
        this.clearLastUsedLensPosition();
    }

    @action
    public setSectorId(sectorId: string): void {
        this.sectorId = sectorId;
    }
}

export type SerializedSectorDatum = {
    sectorId: string;
    containerId: number | null;
    order: number;
    lensData: SerializedLensDatum[]
}