import {BatchDisplayInfo, BatchInitData, BatchManager, LensSide} from "./BatchManager";
import {BatchData, BatchWorkstationModel, JobCallData} from "../../stores/BatchWorkstationStore";
import {action, computed, makeObservable, observable, override} from "mobx";
import {t} from "@lingui/macro";
import {ArBatchNew} from "../../components/ar_coating/workstation/forms/ArBatchNew";
import {ArBatchEdit} from "../../components/ar_coating/workstation/forms/ArBatchEdit";
import {ArBatchSummary} from "../../components/ar_coating/workstation/forms/ArBatchSummary";
import {LensDatum} from "../ar_coating/LensDatum";
import {SectorDatum, SerializedSectorDatum} from "../ar_coating/SectorDatum";

export class ArBatchManager extends BatchManager {
    constructor(workstation: BatchWorkstationModel, batchData: BatchData | null) {
        super(workstation, batchData as BatchData);
        makeObservable(this)
    }

    public newView = ArBatchNew;
    public editView = ArBatchEdit;
    public summaryView = ArBatchSummary

    @action
    public async initialize(data: ArBatchInitData) {
        const fullData = {...this.buildBaseBatchData(data), ...await this.generateInitialBatchDataFromTemplate(data)};

        // save the data first so the UI is not jittery
        await this.saveBatchData(fullData);

        this._data = fullData;
    }

    // DATA ACCESSORS

    @computed
    public get sectorData(): SectorDatum[] {
        const batchData = this.batchData;

        return batchData.sectorData as SectorDatum[];
    }

    @computed
    public get sectorCount(): number {
        return this.sectorData.length;
    }

    @computed
    public get batchData(): ArBatchData {
        const data = this.data;

        return data.batchData as ArBatchData;
    }

    @computed
    public get domeIsComplete(): boolean {
        return this.sectorData.every(sector => sector.isComplete);
    }

    @computed
    public get batchDisplayInfo(): BatchDisplayInfo[] {
        const concaveData = this.batchData.runs.find(run => run.domeSide == 'CC')
        const convexData = this.batchData.runs.find(run => run.domeSide == 'CX')

        const displayInfo = [
            {label: t`Dome Lens Side`, value:  (concaveData && convexData) ? 'CC and CX' : (convexData ? 'Only CX' : 'Only CC')}
        ]

        if(convexData){
            displayInfo.push({label: t`Recipe (CX)`, value: convexData.recipeName});
        }

        if(concaveData){
            displayInfo.push({label: t`Recipe (CC)`, value: concaveData.recipeName});
        }

        displayInfo.push({label: t`Total Lens Count`, value: this.sectorData.map(sector => sector.completedHolesCount).reduce((acc, val)=> acc + val, 0).toString()});
        displayInfo.push({label: t`Total Skipped Holes`, value: this.sectorData.map(sector => sector.emptyHolesCount).reduce((acc, val)=> acc + val, 0).toString()});

        return displayInfo;
    }

    @computed
    public get currentSector(): SectorDatum {
        return this.sectorData[this.currentSectorIndex];
    }

    @computed
    public get currentSectorLensData(): LensDatum[] {
        return this.currentSector.lensData;
    }

    @computed
    public get currentSectorIsLastSector(): boolean {
        return this.currentSectorIndex === this.sectorCount - 1;
    }

    @observable currentSectorIndex = 0;

    // END - DATA ACCESSORS

    // UI action handlers

    @action
    setSectorNumber(value: number): void {
        this.currentSector.clearLastUsedLensPosition();
        this.currentSectorIndex = value;
    }

    protected validateJobData(jobNumber: string, jobData: JobCallData): string | null {
        this.currentSector.clearLastUsedLensPosition();

        const requiredHoles = (jobData.left ? 1 : 0) + (jobData.right ? 1 : 0);

        if(jobData.left){
            const num = (jobData.left['JOB'] ? jobData.left['JOB'] : jobNumber) as string;
            const lensOtherPos = this.findLensOtherPositionInDome(num, 'L');

            if(lensOtherPos){
                return `Cannot add duplicate lens. Lens number ${num} (side: Left) found in sector ${lensOtherPos.sector}, position ${lensOtherPos.position}.`;
            }
        }

        if(jobData.right){
            const num = (jobData.right['JOB'] ? jobData.right['JOB'] : jobNumber) as string;
            const lensOtherPos = this.findLensOtherPositionInDome(num, 'R');

            if(lensOtherPos){
                return `Cannot add duplicate lens. Lens number ${num} (side: Right) found in sector ${lensOtherPos.sector}, position ${lensOtherPos.position}.`;
            }
        }

        if(!this.currentSector.hasEnoughEmptyHoles(requiredHoles)){
            return t`The current sector does not have enough empty holes to fit all required lenses (2 holes required). Please try again with another sector.`;
        }

        return null;
    }

    @action
    protected handleJobData(jobNumber: string, jobData: JobCallData): void {
        if(jobData.left){
            const num = (jobData.left['JOB'] ? jobData.left['JOB'] : jobNumber) as string;
            this.currentSector.addJobToNextAvailableHole(num, 'L');

        }

        if(jobData.right){
            const num = (jobData.right['JOB'] ? jobData.right['JOB'] : jobNumber) as string;
            this.currentSector.addJobToNextAvailableHole(num, 'R');
        }
    }

    @action
    public callLensAction(action: () => void){
        action();
        this.currentSector.clearLastUsedLensPosition();
        this.setUserLastActionTime();
    }

    @action
    public skipAllHolesInCurrentSector(): void {
        this.currentSector.skipAllRemainingHoles();
        this.setUserLastActionTime();
    }

    @action
    public async restartCurrentSector(): Promise<void> {
        this.currentSector.reset();
        await this.saveCurrentBatchData();
    }

    @action
    public jumpToNextSector(): void {
        this.currentSector.clearLastUsedLensPosition();
        if(this.currentSectorIndex === this.sectorCount - 1){
            return;
        }

        this.currentSectorIndex++;
    }

    @override
    public setBatchId(id: string): void {
        this.data.batchId = id;
        this.sectorData.forEach(sector => {
            sector.setSectorId(`${id}${sector.order}`);
        });
    }

    // END - UI action handlers

    public sectorIdIsUnique(sectorId: string ): boolean {
        return this.sectorData.map(sector => sector.sectorId).filter(id => id == sectorId).length === 0;
    }

    protected generateBatchDataFromSerialized(data: SerializedArBatchData): ArBatchData {
        return {
            runs: data.runs,
            sectorData: data.sectorData.map(sectorData => new SectorDatum(this, sectorData))
        };
    }

    protected beforeFinalize(): void {
        // Generate the run ids before doing the final submit
        this.batchData.runs.forEach(run => run.run_key = `${this.data.batchId}${run.domeSide}`);
    }


    protected serializeBatchData(): SerializedArBatchData | null {
        return {
            runs: this.batchData.runs,
            sectorData: this.batchData.sectorData.map(sector => sector.serialize())
        };
    }

    private async generateInitialBatchDataFromTemplate(data: ArBatchInitData): Promise<{batchData: ArBatchData}> {

        // wrap the batch data in 'batchData' so in the merge with base it overwrites the base batch data.
        return {
            batchData: {
                runs: data.runs,
                sectorData: Array.from({length: data.sectorCount}, (_, i) => i + 1).map((sectorOrder) => new SectorDatum(this,{
                        sectorId: `${data.batchId}${sectorOrder}`,
                        containerId: null,
                        order: sectorOrder,
                        lensData: []
                    })
                )
            }
        };
    }

    private findLensOtherPositionInDome(jobNumber: string, side: LensSide): null | {sector: number, position: number} {
        for(const sector of this.sectorData){
            const exists = sector.lensData.find(lensDatum => lensDatum.jobInfo && lensDatum.jobInfo.jobNumber === jobNumber && lensDatum.jobInfo.side === side);
            if(exists){
               return {sector: sector.order, position: exists.position};
            }
        }
        return null;
    }
}

export type ArBatchData = {
    runs: ArBatchRunData[];
    sectorData: SectorDatum[];
}

type ArBatchInitData = {
    runs: ArBatchRunData[];
    sectorCount: number
} & BatchInitData

export type ArBatchRunData = {
    run_key: string;
    recipeName: string;
    recipeCode: string;
    domeSide: DomeSide;
}

export type SerializedArBatchData = {
    runs: ArBatchRunData[];
    sectorData: SerializedSectorDatum[]
}

export type DomeSide = 'both' | 'CX' | 'CC'