import {action, computed, makeObservable, observable} from "mobx";
import {BatchData, BatchWorkstationModel, BatchWorkstationState, JobCallData} from "../../stores/BatchWorkstationStore";
import {BatchProcessModel, BatchProcessStore} from "../../stores/BatchProcessStore";
import moment, {Moment} from "moment";
import * as React from "react";
import {BatchContainerModel, BatchContainerStore} from "../../stores/BatchContainerStore";

export abstract class BatchManager {
    @observable
    public _data: BatchData | null;

    @observable
    public lastSuccessfulSaveTime: Moment; // use for UI

    private lastAttemptedSaveTime: Moment; // use for auto-save; even if a save is not successful, wait to try again
    protected lastUserActionTime: Moment; // do not bother saving if last user action is past last successful save

    @observable
    public controlsDisabled: boolean;

    @observable
    public displayBatchSummary: boolean;

    @observable
    protected workstation: BatchWorkstationModel;

    private autoSaveRef:  NodeJS.Timer;

    protected constructor(workstation: BatchWorkstationModel, baseBatchData: BatchData | null) {
        makeObservable(this);

        this.lastSuccessfulSaveTime = moment();
        this.lastAttemptedSaveTime = moment();
        this.lastUserActionTime = moment();

        this.workstation = workstation;

        if(baseBatchData){
            baseBatchData.batchData = this.generateBatchDataFromSerialized(baseBatchData.batchData);
        }

        this._data = baseBatchData;

        this.controlsDisabled = false;
        this.displayBatchSummary = false

        this.autoSaveRef = setInterval(async () => {
            await this.autoSave();
        }, BatchManager.AUTO_SAVE_FREQUENCY);
    }

    public abstract newView: React.FunctionComponent;
    public abstract editView: React.FunctionComponent;
    public abstract summaryView: React.FunctionComponent<BatchSummaryProps>;

    public abstract initialize(data: BatchInitData): void;

    public abstract get batchData(): any;

    public abstract get batchDisplayInfo(): BatchDisplayInfo[];

    protected abstract validateJobData(jobNumber: string, jobData: JobCallData): string | null;

    protected abstract handleJobData(jobNumber: string, jobData: JobCallData): void;

    protected abstract generateBatchDataFromSerialized(serializedBatchData: any): any;

    protected abstract serializeBatchData(): any;

    protected abstract beforeFinalize(): void;

    // WORKSTATION SETTING ACCESSORS

    @computed
    public get isLoadingSettings(): boolean {
        return BatchProcessStore.isLoading
    }

    @computed
    public get availableContainers(): BatchContainerModel[] {
        const containerModels = BatchContainerStore.getAll();
        return containerModels.filter(container => this.workstation.batch_container_ids.includes(container.id)) as unknown as BatchContainerModel[]
    }

    @computed
    public get processCode() {
       const process = BatchProcessStore.models.find(p => p.getId() === (this.workstation.batch_process_id as unknown as string)) as BatchProcessModel;

       if(!process){
           throw `Unknown process for process id ${this.workstation.batch_process_id}`;
       }

       return (process as BatchProcessModel).batch_process_code;
    }

    @action
    public getNewBatchId(): string {
        return this.workstation.generateBatchId();
    }

    // END - WORKSTATION SETTING ACCESSORS


    // DATA HELPERS

    @computed
    get state(): BatchManagerState {
        if(this.workstation.state === 'editing'){
            if(this.displayBatchSummary){
                return 'summary';
            }
            if(this._data === null){
                return 'new';
            }
        }

        return this.workstation.state;
    }

    @computed
    public get data(): BatchData {
        if(!this._data){
            throw 'Batch Data (base) not initialized';
        }

        return this._data;
    }

    @computed
    public get batchSubmitSuccess(): null | string {
        return this.workstation.last_save_response;
    }

    @computed
    public get batchSubmitError(): null | string {
        return this.workstation.last_save_response;
    }

    @computed
    public get serializedBatchData(): BatchData {
        return {
            processCode: this.data.processCode,
            version: this.data.version,
            batchId: this.data.batchId,
            startedAtTime: this.data.startedAtTime,
            lastUpdateAtTime: this.data.lastUpdateAtTime,
            batchData: this.batchData ? this.serializeBatchData() : null
        }
    }

    protected buildBaseBatchData(data: BatchInitData): BatchData {
        return {
            processCode: this.processCode,
            version: 1,
            batchId: data.batchId,
            startedAtTime: moment(),
            lastUpdateAtTime: moment(),
            batchData: {}
        }
    }

    // END - DATA HELPERS

    // ACTIONS

    public setUserLastActionTime(): void {
        this.lastUserActionTime = moment();
    }

    @action
    public async addJobNumberToBatch(jobNumber: string): Promise<null | string> {
        this.controlsDisabled = true;
        this.setUserLastActionTime();

        try {
            const data = await this.workstation.getDataForJob(jobNumber);

            const validationError = this.validateJobData(jobNumber, data);

            if(validationError){
                this.controlsDisabled = false;
                return validationError;
            }

            this.handleJobData(jobNumber, data);

            this.controlsDisabled = false;
        }catch (e) {
            this.controlsDisabled = false; // ensure controls are unlocked
            throw e;
        }

        return null;
    }

    @action
    public async saveCurrentBatchData(): Promise<void> {
        await this.saveBatchData(this.serializedBatchData);
    }

    @action
    public async resetBatch(): Promise<void> {
        await this.saveBatchData(null);

        this._data = null;
    }

    @action
    public setShowBatchSummary(show: boolean): void {
        this.displayBatchSummary = show;
    }

    @action
    protected async saveBatchData(data: BatchData | null) {
        this.lastAttemptedSaveTime = moment();
        try {
            this.controlsDisabled = true;

            await this.workstation.saveBatchData(data);

            this.lastSuccessfulSaveTime = moment();

            this.controlsDisabled = false;
        }catch (e) {
            this.controlsDisabled = false; // ensure controls are unlocked
            throw e;
        }
    }

    @action
    public async confirmAndFinalizeCurrentBatch(): Promise<void> {
        this.beforeFinalize();

        await this.workstation.createRxDataBatch(this.serializedBatchData)
    }

    @action
    public async returnToEditScreen(): Promise<void> {
        this.setShowBatchSummary(false);
        await this.workstation.setToEditState();
    }

    @action
    public setBatchId(id: string): void {
        this.data.batchId = id;
    }

    // END - ACTIONS

    public stopAutoSave(): void {
        clearInterval(this.autoSaveRef);
    }

    @action
    private async autoSave(): Promise<void> {
        if(moment.duration(moment().diff(this.lastAttemptedSaveTime)).asMinutes() < 2){
            return;
        }

        if(this.lastUserActionTime <= this.lastSuccessfulSaveTime){
            return;
        }

        if(this.state !== "editing"){
            return;
        }

        try{
            await this.saveCurrentBatchData();
        }catch(e){
            console.error(`Auto Save Failed: ${e}`);
        }
    }

    private static AUTO_SAVE_FREQUENCY = (1000 * 60 * 2) + (10 * 1000);
}

export type BatchInitData = {
    batchId: string;
}

export type BatchDisplayInfo = {
    label: string;
    value: string;
}

type BatchSummaryProps = {
    data: BatchData;
    manager: BatchManager;
}

export type LensSide = 'L' | 'R';

type BatchManagerState = 'new' | 'summary' | BatchWorkstationState