import {action, computed, makeObservable, observable, reaction, when} from "mobx";
import {Model} from "@rx-suite/data";
import {BatchContainerComponentModel} from "../../stores/BatchContainerComponentStore";

export abstract class CanvasRenderer {
    protected constructor() {
        makeObservable(this);
        reaction(() => this.selectedComponent?.viewModelAsJSON, () => {
            when(() => this.ctx, () => {
                this.draw();
            });
        });
    }

    @observable selectedComponent: CanvasComponent | undefined;

    @action.bound setSelectedComponent(c: CanvasComponent | undefined) {
        this.selectedComponent = c?.viewModel;
    }

    @observable state: LayoutEditState = 'done';

    protected lastMousePosition = CanvasRenderer.MOUSE_INITIAL_POSITION;

    @observable
    protected _layout: undefined | CanvasLayout;
    get layout(): CanvasLayout {
        return (this._layout as unknown) as CanvasLayout;
    }

    @observable
    protected _canvas: HTMLCanvasElement | null = null;
    get canvas(): HTMLCanvasElement {
        return (this._canvas as unknown) as HTMLCanvasElement;
    }

    get ctx(): CanvasRenderingContext2D {
        return this.canvas.getContext('2d') as CanvasRenderingContext2D;
    }

    @computed
    get ready(): boolean {
        return this._canvas != null;
    }

    @action.bound
    setLayout(layout: CanvasLayout): void {
        this._layout = layout;
        this.afterSetLayout();
        this.draw();
    }

    // provide hook for child when layout is set to update any data before a re-draw
    protected abstract afterSetLayout(): void;

    @action.bound
    setCanvasRef(c: any): void {
        if (c) {
            this._canvas = c;
        }
    }

    public draw(): void {
        if (!this.ready) return;

        this.clearCanvas();
        this.drawBorder();

        this.handleDraw();
    }

    // Let the child classes handle the drawing of the elements
    protected abstract handleDraw(): void;

    //// UTILITY FOR CANVAS SPACE CONVERSION

    protected convertToCanvasSpace(x: number, y: number): CanvasComponentPosition {
        const rect = this.canvas.getBoundingClientRect();
        x = x - rect.left;
        y = y - rect.top;
        const xScale = this.canvas.width / this.canvas.clientWidth;
        const yScale = this.canvas.height / this.canvas.clientHeight;
        return {
            x: x * xScale,
            y: y * yScale
        }
    }

    protected eventInputPosition(e: any): CanvasComponentPosition {
        if (e.touches) {
            return this.convertToCanvasSpace(e.touches[0].clientX, e.touches[0].clientY);
        }
        return this.convertToCanvasSpace(e.clientX, e.clientY);
    }

    //// DRAW FUNCTIONS

    protected drawText(text: string, pos: CanvasComponentPosition): void {
        this.ctx.font = '600 ' + '3' + 'rem Segoe UI';
        this.ctx.fillStyle = "white";
        this.ctx.textAlign = "center";
        this.ctx.textBaseline = "middle";
        this.ctx.fillText(text, pos.x , pos.y, 30);
    }

    protected drawLine(start: CanvasComponentPosition, end: CanvasComponentPosition, opts: RenderOptions = {}) {
        this.ctx.save();
        this.setRenderOptions(opts);

        this.ctx.beginPath();
        this.ctx.moveTo(start.x, start.y)
        this.ctx.lineTo(end.x, end.y);
        this.ctx.stroke();
        this.ctx.closePath();

        this.ctx.restore();
    }

    protected drawRectangle(pos: CanvasComponentPosition, width: number, height: number, opts: RenderOptions = {}) {
        this.ctx.save();
        this.setRenderOptions(opts);

        this.ctx.beginPath();
        if (opts.fillColor) {
            if (opts.strokeColor) {
                this.ctx.fillRect(pos.x, pos.y, width, height);
                this.ctx.rect(pos.x, pos.y, width, height);
            } else {
                this.ctx.fillRect(pos.x, pos.y, width, height);
            }
        } else {
            this.ctx.rect(pos.x, pos.y, width, height);
        }
        this.ctx.stroke();
        this.ctx.closePath();

        this.ctx.restore();
    }

    protected drawTriangle(pos: CanvasComponentPosition, size: number, opts: RenderOptions = {}) {
        this.ctx.save();
        this.setRenderOptions(opts);

        this.ctx.translate(pos.x + size, pos.y);
        this.ctx.beginPath();
        this.ctx.moveTo(0, 0);
        this.ctx.lineTo(size, size * 2);
        this.ctx.lineTo(-size, size * 2);
        this.ctx.lineTo(0, 0);
        this.ctx.stroke();
        this.ctx.fill();
        this.ctx.closePath();

        this.ctx.restore();
    }

    protected drawExclamation(pos: CanvasComponentPosition, size: number, opts: RenderOptions = {}) {
        this.ctx.save();
        this.setRenderOptions(opts);

        this.ctx.translate(pos.x + size, pos.y + (size / 4));
        this.ctx.beginPath();
        this.ctx.arc(0, 0, size / 4, 0, Math.PI, true);
        this.ctx.arc(0, size + (size / 4), size / 9, Math.PI, 0, true);
        this.ctx.lineTo(size / 4, 0);
        this.ctx.stroke();
        this.ctx.fill();
        this.ctx.closePath();
        this.ctx.beginPath();
        this.ctx.arc(0, size + size / 1.6, size / 8, 0, Math.PI * 2);
        this.ctx.stroke();
        this.ctx.fill();

        this.ctx.restore();
    }

    protected drawCircle(pos: CanvasComponentPosition, size: number, opts: RenderOptions = {}): void {
        this.ctx.save();
        this.setRenderOptions(opts);

        this.ctx.beginPath();
        this.ctx.arc(pos.x + (size / 2), pos.y + (size / 2), size / 2, 0, Math.PI * 2, false);
        this.ctx.fill();
        this.ctx.closePath();

        this.ctx.restore();
    }

    protected drawOval(pos: CanvasComponentPosition, radiusX: number, radiusY: number, opts: RenderOptions = {}): void {
        this.ctx.save();
        this.setRenderOptions(opts);

        this.ctx.beginPath();
        this.ctx.ellipse(pos.x + radiusX, pos.y + radiusY, radiusX, radiusY, 0, 0,Math.PI * 2, false);
        this.ctx.fill();
        this.ctx.closePath();

        this.ctx.restore();
    }

    protected drawPartialCircle(startPoint: number, sectorSize: number, size: number, opts: RenderOptions = {}): void {
        this.ctx.save();

        const midPoint = (size / 2);

        this.setRenderOptions(opts);
        this.ctx.beginPath();
        this.ctx.moveTo(midPoint, midPoint);
        this.ctx.arc(midPoint,  midPoint, midPoint, startPoint, startPoint + sectorSize, false);
        if(!opts.fillColor){
            this.ctx.stroke();
        }
        else {
            this.ctx.fill();
        }
        this.ctx.closePath();

        this.ctx.restore();
    }

    protected drawImage(c: CanvasComponent): void {
        // if the image is already loaded, render it now
        if (c.image) {
            this.ctx.drawImage(c.image, c.x, c.y, c.width, c.height)
            // Image was already saved with the back end, load it from there
        } else if (c.image_url) {
            c.image = new Image();
            c.image.src = c.image_url;
            c.image.onload = () => this.draw();
            // Image exists only in the front-end, load it from a blob
        } else {
            const reader = new FileReader();
            reader.onload = (b: ProgressEvent<FileReader>) => {
                if (!b.target) return;
                c.image = new Image();
                c.image.src = b.target.result as string;
                c.image.onload = () => {
                    if (!c.image) {
                        return;
                    }
                    const imageWidth = c.image.width as number;
                    const imageHeight = c.image.height as number;

                    c.width = imageWidth > this.canvas.width ? this.canvas.width - 50 : imageWidth;
                    c.height = imageHeight > this.canvas.height ? this.canvas.height - 50 : imageHeight;
                    c.aspect_ratio = c.width / c.height;
                }
            };
            if (c.file) {
                reader.readAsDataURL(c.file as Blob);
            }
        }
    }

    private clearCanvas(): void {
        this.ctx.clearRect(0, 0, this.canvas.width, this.canvas.height);
    }

    private setRenderOptions(opts: RenderOptions) {
        if (opts.fillColor) this.ctx.fillStyle = opts.fillColor;
        if (opts.lineWidth) this.ctx.lineWidth = opts.lineWidth;
        if (opts.strokeColor) this.ctx.strokeStyle = opts.strokeColor;
    }

    private drawBorder(): void {
       // this.drawRectangle({x: 0, y: 0}, this.canvas.width, this.canvas.height);
    }

    protected static MOUSE_INITIAL_POSITION = {x: 0, y: 0};
}

type RenderOptions = {
    strokeColor?: string;
    lineWidth?: number;
    fillColor?: string;
}

export interface CanvasComponentPosition {
    x: number;
    y: number;
}

export type CanvasLayout = {
    components: BatchContainerComponentModel[];
    current_components: BatchContainerComponentModel[]; // Returns references to view models of components
    current_ordered_component: BatchContainerComponentModel[]; // Returns references to view models of components in render order
    component_count: number; deleteComponent: (component: any) => void;
} & Model

export type CanvasComponent = {
    x: number;
    y: number;
    height: number;
    width: number;
    aspect_ratio: number;
    z_index: number;
    component_type: string;
    image?: CanvasImageSource | undefined;
    image_url?: string | null;
    file?: Blob | null;
    canvas_position: CanvasComponentPosition;
    pointOverlapsComponentCorner : (component: any) => boolean;
    pointOverlapsComponentSide : (component: any) => boolean;
    pointOverlapsComponentBottom : (component: any) => boolean;
    pointOverlapsComponent: (component: any) => boolean;
} & Model


type LayoutEditState = 'done' | 'mouse_down' | 'dragging' | 'saving' | 'resizing' | 'resizingX' | 'resizingY';