import {CanvasRenderer} from "../CanvasRenderer";
import {BatchContainerViewRenderer} from "./BatchContainerViewRender";
import {computed, makeObservable} from "mobx";
import {BatchContainerComponentModel} from "../../../stores/BatchContainerComponentStore";

export class BatchContainerEditRenderer extends BatchContainerViewRenderer {
    constructor() {
        super();
        makeObservable(this);
    }

    @computed
    get hasContainerImage(): boolean {
        return this.layout.current_components.find(component => component.component_type === 'batch_image') !== undefined;
    }

    public deleteSelectedComponent(): void {
        if (!this.selectedComponent) return;
        this.layout.deleteComponent(this.selectedComponent);
        this.setSelectedComponent(undefined);
    }

    protected handleDraw(): void {
        // Use the view model to render the positions
        this.ensureComponentsPositionData();

        this.layout.current_ordered_component.forEach(c => {
            switch (c.component_type) {
                case 'batch_position':
                    this.drawBatchPosition(c);
                    break;
                case 'batch_image':
                    this.drawImage(c);
                    break;
            }
        });
        this.drawSelectedComponentBoundaries();
    }

    protected afterSetLayout(): void {
        // Nothing to do here in edit mode
    }

    private drawSelectedComponentBoundaries(): void {
        if (!this.selectedComponent) return;
        const c = this.selectedComponent;
        this.drawRectangle(c.canvas_position, c.width, c.height, BatchContainerEditRenderer.SELECTED_COMPONENT_BORDER_STYLE);
        this.drawLine({x: c.x + 0, y: c.y + c.height - 15}, {x: c.x + c.width, y: c.y +c.height - 15}, BatchContainerEditRenderer.SELECTED_COMPONENT_BORDER_STYLE);
        this.drawLine({x: c.x + c.width - 15, y: c.y + 0}, {x: c.x + c.width - 15, y: c.y + c.height}, BatchContainerEditRenderer.SELECTED_COMPONENT_BORDER_STYLE);
    }

    private static SELECTED_COMPONENT_BORDER_STYLE = {strokeColor: "red", lineWidth: 1};

    //// Canvas Action Handlers

    public handleMouseMove(e: any): void {
        if (e.touches) e.preventDefault();
        const point = this.eventInputPosition(e);
        const c = this.selectedComponent;
        if (c && (this.state === 'mouse_down' || this.state === 'dragging')) {
            if (this.state === 'mouse_down') {
                if (point.x > c.x && point.x < c.x + c.width &&
                    point.y > c.y && point.y < c.y + c.height) {
                    this.state = 'dragging';
                }
            } else if (this.state === 'dragging') {
                c.x = Math.floor(c.x + (point.x - this.lastMousePosition.x));
                c.y = Math.floor(c.y + (point.y - this.lastMousePosition.y));
                this.lastMousePosition = point;
            }
        } else if (c && this.state === 'resizing') {
            const x = c.x + c.width;
            const xOffset = point.x - x;
            c.width += xOffset;
            c.height += xOffset / c.aspect_ratio;

            this.ensureSelectedComponentMinWidth();
            this.ensureSelectedComponentMinHeight();
        }
        else if (c && this.state === 'resizingX') {
            const x = c.x + c.width;
            const xOffset = point.x - x;
            c.width += xOffset;
            this.ensureSelectedComponentMinWidth();
        }
        else if (c && this.state === 'resizingY') {
            const y = c.y + c.height;
            const yOffset = point.y - y;
            c.height += yOffset;
            this.ensureSelectedComponentMinHeight();
        }
    }

    public handleMouseUp(e: any): void {
        if (this.state === 'saving') return;
        if (this.state === 'mouse_down') {
            const point = this.eventInputPosition(e);
            const comp = this.layout.current_ordered_component.reverse().find(c => {
                return point.x > c.x && point.x < c.x + c.width && point.y > c.y && point.y < c.y + c.height;
            });
            if (comp) {
                this.setSelectedComponent(comp);
            } else {
                this.setSelectedComponent(undefined);
            }
        }
        this.state = 'done';
        this.lastMousePosition = CanvasRenderer.MOUSE_INITIAL_POSITION;
    }

    public handleMouseDown(e: any): void {
        if (this.state === 'saving') return;
        this.state = 'mouse_down';
        const point = this.eventInputPosition(e);
        this.lastMousePosition = point;
        const c = this.selectedComponent;
        if (c) {
            if (c.pointOverlapsComponentCorner(point)) {
                this.state = 'resizing';
            }
            else if(c.pointOverlapsComponentBottom(point)){
                this.state = 'resizingY';
            }
            else if(c.pointOverlapsComponentSide(point)){
                this.state = 'resizingX';
            }
        }
    }

    public handleMouseLeave(): void {
        if (this.state === 'saving') return;
        this.state = 'done';
    }

    public handleKeyDown(e: any): void {
        if (e.keyCode === 46) {
            this.deleteSelectedComponent();
        }
    }

    private ensureSelectedComponentMinHeight(): void {
        const component = this.selectedComponent as BatchContainerComponentModel;
        component.height = Math.floor(component.height < BatchContainerViewRenderer.BATCH_POSITION_MIN_HEIGHT ? BatchContainerViewRenderer.BATCH_POSITION_MIN_HEIGHT : component.height);
    }

    private ensureSelectedComponentMinWidth(): void {
        const component = this.selectedComponent as BatchContainerComponentModel;
        component.width = Math.floor(component.width < BatchContainerViewRenderer.BATCH_POSITION_MIN_WIDTH ? BatchContainerViewRenderer.BATCH_POSITION_MIN_WIDTH : component.width);
    }

}
