import * as React from "react";
import {ButtonField, Conditional, IfTrue, ModelDcsSelect, ModelForm, ModelInput, ModelSelect} from "@rx-suite/ui";
import {observer} from "mobx-react-lite";
import {BatchProcessStore} from "../../stores/BatchProcessStore";
import {Box, Divider, LinearProgress, makeStyles, Theme, Typography} from "@material-ui/core";
import {BatchWorkstationModel} from "../../stores/BatchWorkstationStore";
import {t, Trans} from "@lingui/macro";
import {useBatchingProcessOptions} from "../../components/hooks/useBatchingProcessOptions";
import {Delete} from "@material-ui/icons";
import {red} from "@material-ui/core/colors";
import {useModalHandler} from "../../components/hooks/useModalHandler";
import {useSnackbar} from "notistack";
import {WarningDialog} from "../../components/batch_workstations/dialogs/WarningDialog";
import {BatchContainerStore} from "../../stores/BatchContainerStore";
import {WorkstationNameField} from "../../components/generic/WorkstationNameField";

type BatchWorkstationFormProps = {
    model: BatchWorkstationModel;
};

const useStyles = makeStyles((theme: Theme) => ({
        resetButton: {
            background: red[600]
        }
    })
);

export const BatchWorkstationForm: React.FunctionComponent<BatchWorkstationFormProps> = observer(({model}) => {
    const classes = useStyles();
    const {enqueueSnackbar} = useSnackbar();

    BatchContainerStore.getAll();

    const [resetBatchDialogOpen, handleCloseResetBatchDialog, handleOpenResetBatchDialog] = useModalHandler();
    const [isResetting, setIsResetting] = React.useState<boolean>(false);

    const processOptions = React.useMemo(()=> {
        return useBatchingProcessOptions();
    }, [BatchProcessStore.models]);

    const availableTemplates = React.useMemo(()=> {
        return BatchContainerStore.models
            .filter((container) => { return container.batch_process_id === model.viewModel.batch_process_id; })
            .map((container) => {
                return {label: container.name, value: container.getId()}
            });
    }, [BatchContainerStore.models, model.viewModel.batch_process_id]);

    const handleReset = React.useCallback(async () => {
        handleCloseResetBatchDialog();
        try {
            setIsResetting(true);
            await model.saveBatchData(null);
            enqueueSnackbar(`${t`Current batch data deleted`}`, {variant: "success"})
        }catch (e) {
            console.error(e);
            enqueueSnackbar(`${t`Error while resetting batch!`}`, {variant: "error"})
        }
        setIsResetting(false);
    }, []);

    if (BatchProcessStore.isLoading || BatchContainerStore.isLoading) {
        return <LinearProgress/>;
    }

    return (
        <ModelForm model={model} suppressMakeCopy suppressImportExport>
            <Conditional condition={!model.isNew}>
                <ButtonField
                    startIcon={<Delete/>}
                    text={t`Delete Current Batch Data`}
                    className={classes.resetButton}
                    onClick={handleOpenResetBatchDialog}
                    loading={isResetting}
                />
                <WarningDialog
                    warningMessage={t`This action delete all the data of the current batch.`}
                    open={resetBatchDialogOpen}
                    onConfirm={handleReset}
                    handleClose={handleCloseResetBatchDialog}
                />
            </Conditional>
            <Divider/>
            <WorkstationNameField name={'name'}/>
            <ModelSelect
                name={'batch_process_id'}
                options={processOptions}
                label={`Process`}
                multiple={false}
            />
            <ModelSelect
                         name={'batch_container_ids'}
                         label={`Batching Templates`}
                         options={availableTemplates}
                         multiple={true}
            />
            <Box mt={1}>
                <Divider/>
                <Typography variant={'h6'}>
                    <Trans>
                        Job Data Source
                    </Trans>
                </Typography>
            </Box>
            <ModelSelect name={'data_source'} options={[
                    {label: 'LMS', value: 'lms'},
                    //{label: 'Rx-Data', value: 'rx_data'} // Temporarily disabled until implemented
                ]}
            />
            <Conditional condition={model.viewModel.data_source === 'lms'}>
                <IfTrue>
                    <ModelDcsSelect
                        name={'external_dcs_id'}
                    />
                </IfTrue>
            </Conditional>
        </ModelForm>
    );
})