import {attribute, DataStore, Model, model, Types} from '@rx-suite/data';
import * as yup from 'yup';
import { t } from "@lingui/macro"
import {BatchContainerComponentModel} from "./BatchContainerComponentStore";
import {action, computed, observable, override, runInAction} from "mobx";

@model({
    transportLayer: "/api/batch_containers",
        permissionTarget: "BatchContainer",
        routingInfo: {
            create: '/batch_containers/create',
            read: '/batch_containers/:id/edit',
            update: '/batch_containers/:id/edit'
        }
})

export class BatchContainerModel extends Model {
    static types: Types = {
        components: 'array'
    };

    static relationships = {
        components: BatchContainerComponentModel
    };

    @attribute()
    id: number = -1;

    @attribute({
        label: t`Name`,
        validation: yup.string().min(3).required(t`Sector name is required.`)
    })
    name: string = '';

    @attribute({
        label: t`Batch Process Id`,
        validation: yup.number()
    })
    batch_process_id: number = 0;

    // Don't load the components from the index page, wait for a show
    @attribute()
    loaded_full_model: Boolean = false

    @observable
    is_fetching_full_model: boolean = false;

    @computed
    get current_components(): BatchContainerComponentModel[] {
        if(!this.isNew){
            if(this.is_fetching_full_model){
                return [];
            }

            if(!this.loaded_full_model){
                this.fetchFullModel();
                return [];
            }
        }

        return this.components.map(c => c.viewModel);
    }

    @computed
    get current_ordered_component(): BatchContainerComponentModel[] {
        // Return the components ordered by z-index so that selection is always from the top
        return this.current_components.map(c => c.viewModel).sort((a, b) => a.z_index - b.z_index);
    }

    @attribute()
    private components: BatchContainerComponentModel[] = []

    @computed
    get component_count(): number {
        return this.current_components.filter((obj) => obj.component_type === 'batch_position').length;
    }

    @action.bound
    deleteComponent(component: BatchContainerComponentModel) {
        this.components.splice(this.current_components.map(c => c._tmpId).indexOf(component._tmpId), 1);
    }

    @override
    async commit(): Promise<void> {
        //save all the component models
        runInAction(() => {
            this._state = 'updating';
        });
        this.components.forEach(c => c.viewModel.submit());
        this.viewModel.submit();
        const formData = new FormData();
        formData.append('batch', JSON.stringify(this.asJson));
        this.current_components.filter(c => c.component_type == "batch_image").forEach(i => {
            if (i.file) {
                formData.append('container_image', i.file as Blob);
            }
        });
        const response = await BatchContainerStore.transportLayer.fetch(`/save`, {
            method: 'POST',
            body: formData
        })
        runInAction(() => {
            this._state = 'done';
        });
        return BatchContainerStore.transportLayer.parseJson(response);
    }

    @action
    async fetchFullModel(): Promise<BatchContainerModel> {
        if(this.loaded_full_model || this.isNew){
            return this;
        }

        this.is_fetching_full_model = true;

        const resp = await fetch(`${this.store.transportLayer.base}/${this.id}`, {
            method: "GET",
            headers: {'Content-Type': 'application/json'}
        })
        const data = await resp.json();

        if (!resp.ok) {
            throw data.error;
        }
        const model = this.store.createModel(data);

        return this.store.setModel(model, true) as BatchContainerModel;
    }

    constructor() {
        super();
        super.setup();
    }
}

export const BatchContainerStore = BatchContainerModel.store as DataStore<BatchContainerModel>;