import {attribute, DataStore, Model, model} from "@rx-suite/data";
import {action} from "mobx";
import {BatchData} from "./BatchWorkstationStore";
import moment from "moment";
import {BatchProcessCode} from "./BatchProcessStore";

type RxDataBatchData = {
    processCode: string;
    batchId: string;
    createdAt: string;
    batchData: any;
};

type RxDataBatchDataError = {
    error: string
};

@model({
    transportLayer: "/api/batches",
    routingInfo: {
        read: '/batches/:id',
        update: '/batches/:id/edit'
    }
})
export class BatchModel extends Model {

    @attribute()
    batch_id: number = -1

    @attribute()
    process_code: BatchProcessCode = 'ar_coating';

    @attribute()
    created_at: string | null = null;

    @attribute()
    updated_at: string | null = null;


    @action
    async getRxDataBatch(): Promise<BatchData | RxDataBatchDataError> {
        const resp = await fetch(`${BatchStore.transportLayer.base}/get_rx_data_batch/${this.batch_id}`, {
            method: "GET",
            headers: {'Content-Type': 'application/json'}
        });

        const rxDataBatchData = await resp.json();

        if (rxDataBatchData && rxDataBatchData.error) {
            // data is of type RxDataBatchDataError
            return {
                error: rxDataBatchData.error
            };
        }

        // data is of type RxDataBatchData
        const batchData = convertRxDataBatchDataToBatchData(rxDataBatchData);

        return batchData;
    }


    constructor() {
        super();
        super.setup(); // This registers the observable attributes and viewModel
    }
}

export async function getBatchByJobNumber(processId: number, jobNumber: string): Promise<{ batchId: string } | RxDataBatchDataError> {
    const resp = await BatchStore.transportLayer.fetch(`/get_batch_by_job_number?process_id=${processId}&job_number=${jobNumber}`);
    const rxDataBatchData = await resp.json();

    if (rxDataBatchData && rxDataBatchData.error) {
        // data is of type RxDataBatchDataError
        return {
            error: rxDataBatchData.error
        };
    }

    return { batchId: rxDataBatchData.batch_id };
}

function convertRxDataBatchDataToBatchData(rxDataBatchData: RxDataBatchData): BatchData {
    return {
        batchId: rxDataBatchData.batchId,
        processCode: rxDataBatchData.processCode,
        version: -1, // this version will not be used
        startedAtTime: moment(rxDataBatchData.createdAt),
        lastUpdateAtTime: moment(rxDataBatchData.createdAt),
        batchData: rxDataBatchData.batchData
    };
}


export const BatchStore = BatchModel.store as unknown as DataStore<BatchModel>;