import {attribute, DataStore, Model, model} from "@rx-suite/data";
import {t} from "@lingui/macro";
import * as yup from "yup";
import {action} from "mobx";
import moment, {Moment} from "moment";

@model({
    transportLayer: "/api/batch_workstations",
    permissionTarget: "BatchWorkstation",
    routingInfo: {
        create: '/batch_workstations/create',
        read: '/batch_workstations/:id/edit',
        update: '/batch_workstations/:id/edit'
    }
})

export class BatchWorkstationModel extends Model {
    @attribute({
        label: t`Id`,
        validation: yup.number()
    })
    id: number = 0;

    @attribute({
        label: t`Workstation Name`,
        validation: yup.string().required(t`Workstation name is required.`)
    })
    name: string = '';

    @attribute({
        label: t`Batch Process Id`,
        validation: yup.number().required().typeError(t`Workstation process is required.`)
    })
    batch_process_id: number | null = null;

    @attribute({
        label: t`Batching Templates`,
        validation: yup.array().min(1, "At least one template must be selected.")
    })
    batch_container_ids: number[] = []

    @attribute({
        label: t`Data Source`,
        validation: yup.string()
    })
    data_source: 'lms' | 'rx_data' = 'lms';

    @attribute({
        label: t`External DCS Id`,
        validation: yup.number().nullable()
    })
    external_dcs_id: number | null = null;

    @attribute({})
    state: BatchWorkstationState = 'editing';

    @attribute({})
    last_save_response: 'string' | null = null;

    @attribute({
        label: t`Tags`,
        validation: yup.string()
    })
    tags: string = ''; // comma delimited string 'a,b,c'

    @action generateBatchId(): string {
        return `${this.getId()}${moment().format('YYYYMMDDHHmmssSSS')}`
    }

    @action
    async loadCurrentBatchData(): Promise<BatchData | null> {
        return await this.callWorkstationEndpoint({
            method: 'GET',
            endpoint: 'current_batch'
        });
    }

    @action
    async getDataForJob(jobNumber: string): Promise<JobCallData> {
        const resp = await fetch(`${BatchWorkstationStore.transportLayer.base}/${this.id}/job_call`, {
            method: 'POST',
            headers: {'Content-Type': 'application/json'},
            body: JSON.stringify({
                    job: {
                        number: jobNumber
                    }
                }
            )
        });

        const data = await resp.json();

        if (!resp.ok) {
            throw data; // we don't want to swallow job not found errors
        }

        return data;
    }

    @action
    async saveBatchData(batchData: BatchData | null): Promise<void> {
        return await this.callWorkstationEndpoint({
            method: 'PUT',
            endpoint: 'save_current_batch',
            body: JSON.stringify({
                    current_batch: {
                        data: batchData
                    }
                }
            )
        });
    }

    @action
    private async callWorkstationEndpoint(payload: WorkstationCallPayload): Promise<any> {
        const resp = await fetch(`${BatchWorkstationStore.transportLayer.base}/${this.id}/${payload.endpoint}`, {
            method: payload.method,
            headers: {'Content-Type': 'application/json'},
            body: payload.body
        });

        const data = await resp.json();

        if (!resp.ok) {
            throw data.error;
        }

        return data;
    }

    @action
    async createRxDataBatch(batchData: BatchData): Promise<void> {
        return await this.callWorkstationEndpoint({
            endpoint: 'create_external_batch',
            method: "POST",
            body: JSON.stringify({
                batch: batchData
            })
        });
    }

    @action
    async setToEditState(): Promise<void> {
        return await this.callWorkstationEndpoint({
            method: 'PUT',
            endpoint: 'set_state',
            body: JSON.stringify({
                state: 'editing'
            })
        });
    }

    constructor() {
        super();
        super.setup();
    }
}

type WorkstationCallPayload = {
    method: 'GET' | 'POST' | 'PUT';
    endpoint: 'save_current_batch' | 'job_call' | 'current_batch' | 'create_external_batch' | 'set_state';
    body?: any;
}

export type BatchWorkstationState = 'editing' | 'saving' | 'error' | 'completed';

export const BatchWorkstationStore = BatchWorkstationModel.store as DataStore<BatchWorkstationModel>;

export type BatchData = {
    processCode: string;
    version: number;
    batchId: string;
    startedAtTime: Moment;
    lastUpdateAtTime: Moment;
    batchData: any; // each child class must implement its own batchData prop
}

export type JobCallData = {
    left: TagHash | null;
    right: TagHash | null;
}

type TagHash = {
    [name: string] : string | number
}